function W = detsol(Z)

% DETSOL(Z): Do a determined solution of the elevation points in the
% square matrix Z.  Unknown entries are 0.  B is the matrix of
% coefficients, and C the RHS vector.  W is the output matrix of
% elevations.  The unknown entries become the average of their 2,
% 3, or 4 neighbors.

% Time-stamp: </home/wrf/r/matlab/interp/detsol.m, Wed, 24 Jun 1998, 00:41:24 IDT, wrf@speed.ecse.rpi.edu>


t = cputime;

SMALL=0

N=size(Z,1)
fprintf(1,'Input range, lo: %f, hi: %f\n',min(min(Z(find(Z)))),max(max(Z)))

N2=N*N;

C=zeros(N2,1);

[px,py] = find(Z);		% Find indices of +ve elts
np=size(px,1)			% Number of nonzero elts.
[ox,oy] = find(Z==0);		% Find indices of zero elts
nzelt=size(ox,1)		% Number of nonzero elts.

ns = 0;			% # entries so far in the list of sparse matrix B entries.

if  SMALL
   ssize = 4*nzelt+N2
   B = spalloc(N2+nzelt,N2,ssize);
else
   s = zeros(4*nzelt+N2, 3); % The list of nonzero entries of B.
end

NE=0;                   % Number of equations created so far.  Final value: N2.


% Make an equation saying that each known point is equal to its value.

for k = 1:np	
   NE=NE+1;
   i = px(k);
   j = py(k);
   if SMALL
      B(NE, N*j+i-N) = B(NE, N*j+i-N) +  1;
   else
      ns = ns+1;
      s(ns,:) = [NE, N*j+i-N, 1];
   end
   C(NE) = Z(i,j);
end


% Make an equation saying that each unknown point is the average
% of its 4 (or 3 or 2 )neighbors.

for k = 1:nzelt	
   NE=NE+1;		
   i = ox(k);
   j = oy(k);

   if SMALL
      B(NE, N*j+i-N) = B(NE, N*j+i-N) +  4;
      B(NE, N*j+max(i-1,1)-N) = B(NE, N*j+max(i-1,1)-N) -1; 
      B(NE, N*j+min(i+1,N)-N) = B(NE, N*j+min(i+1,N)-N) -1; 
      B(NE, N*max(j-1,1)+i-N) = B(NE, N*max(j-1,1)+i-N) -1; 
      B(NE, N*min(j+1,N)+i-N) = B(NE, N*min(j+1,N)+i-N) -1; 
   else
      ns = ns+1;
      s(ns,:) = [NE, N*j+i-N, 4];
      ns = ns+1;
      s(ns,:) = [NE, N*j+max(i-1,1)-N, -1]; % This uses the fact that 
					% duplicate sparse matrix entries add.
      ns = ns+1;
      s(ns,:) = [NE, N*j+min(i+1,N)-N, -1]; 
      ns = ns+1;
      s(ns,:) = [NE, N*max(j-1,1)+i-N, -1]; 
      ns = ns+1;
      s(ns,:) = [NE, N*min(j+1,N)+i-N, -1]; 
   end

   C(NE) = 0;
end

clear ox oy px py

if ~ SMALL
   B = spconvert(s);
end

whos
clear s

W1 = B\ C;			% Solve it.
W = reshape(W1,N,N);

fprintf(1,'CPU time for detsol: %f\n', cputime-t);
